/*
Author:       Pascal Peter
Copyright:    (C) 2020 Pascal Peter
License:      GNU General Public License version 3
Website:      http://pascal.peter.free.fr
Email:        pascal.peter at free.fr
*/










// VARIABLES D'AFFICHAGE
const BORDER = 15;
let WIDTH, HEIGHT;

// CANVAS
const   canvas  = document.getElementById('canvas'), 
        context = canvas.getContext('2d'), 
        parent = canvas.parentElement;

// POUR LIRE L'IMAGE
const   FILE_INPUT = document.getElementById('fileselect'), 
        FILE_READER = new FileReader();

// POUR L'ANIMATION (T VARIE ENTRE 0 ET 1)
let T = 1;









function draw() {

    // nombre d'images par groupe
    let N = parseInt(document.getElementById('size').value);
    // unité de largeur (correspond à une largeur d'image)
    let UNIT = canvas.width / (5 * N + 5);

    // on récupère les dimensions de l'image initiale
    let myImg = document.getElementById('myImg');
    let imageWidth = myImg.width, 
        imageHeight = myImg.height;
    //console.log(imageWidth + '|' + imageHeight + '|' + imageHeight / imageWidth);
    // on adapte les dimensions de l'image de destination
    let destWidth = UNIT, destHeight = UNIT * imageHeight / imageWidth;
    if (imageHeight / imageWidth > 4) {
        destWidth = destWidth * 4 * imageWidth / imageHeight;
        destHeight = destHeight * 4 * imageWidth / imageHeight;
    }
    // et la hauteur à utiliser
    let H = 1 + Math.trunc(destHeight / UNIT);
    canvas.height = 2 * (H + 1) * UNIT;

    // positions des 5 groupes
    const positions = {
        0: [4 * N + 3, 2 * N + 2], 
        1: [2 * N + 2, 1], 
        2: [1, 3 * N + 3], 
        3: [3 * N + 3, N + 2], 
        4: [N + 2, 4 * N + 4], 
    }

    let yDecal = (2 * T - 1) * (2 * T - 1);
    // nombre de fractions (nombre total d'images moins un)
    let PARTS = 5 * N - 1;

    for (let i = 0; i < 5; i++) {
        for (let j = 0; j < N; j++) {
            let k = i + 5 * j;
            let sy, dx, dy, sourceHeight;

            sy = 0;
            dx = ((positions[i][0] + 0.5 + j) * T + (positions[i][1] + 0.5 + j) * (1 - T)) * UNIT;
            sourceHeight = imageHeight * (PARTS - k) / PARTS;
            destHeight = sourceHeight * destWidth / imageWidth;
            dy = (yDecal + H) * UNIT - destHeight;
            if (sourceHeight > 0)
                context.drawImage(myImg, 0, sy, imageWidth, sourceHeight, dx, dy, destWidth, destHeight);

            sy = imageHeight * (PARTS - k) / PARTS;
            dx = (positions[i][0] + 0.5 + j) * UNIT;
            dy = (H + 1) * UNIT;
            sourceHeight = imageHeight * k / PARTS;
            destHeight = sourceHeight * destWidth / imageWidth;
            if (sourceHeight > 0)
                context.drawImage(myImg, 0, sy, imageWidth, sourceHeight, dx, dy, destWidth, destHeight);

        }
    }

    // dessin des 3 rectangles
    context.lineWidth = '2';
    context.strokeStyle = 'gray';
    //console.log(UNIT + '|' + Math.trunc(UNIT / 2));
    context.font = Math.trunc(UNIT / 2) + 'px Calibri,Geneva,Arial';

    context.strokeRect(
        (T + (3 * N + 3) * (1 - T)) * UNIT, 
        yDecal * UNIT, 
        (2 * N + 1) * UNIT, 
        H * UNIT);
    context.fillText(
        'A', 
        ((2 * N + 1.5) * T + (5 * N + 3.5) * (1 - T)) * UNIT, 
        (yDecal + 0.6) * UNIT);
    context.strokeRect(
        ((2 * N + 2) * T + (1 - T)) * UNIT, 
        yDecal * UNIT, 
        (3 * N + 2) * UNIT, 
        H * UNIT);
    context.fillText(
        'B', 
        ((5 * N + 3.5) * T + (3 * N + 2.5) * (1 - T)) * UNIT, 
        (yDecal + 0.6) * UNIT);

    context.strokeRect(UNIT, (H + 1) * UNIT, (5 * N + 3) * UNIT, H * UNIT);
    context.fillText('C', 1.2 * UNIT, (2 * H + 0.8) * UNIT);

}

function onWindowResize() {
    WIDTH = window.innerWidth;
    HEIGHT = window.innerHeight;
    //console.log(WIDTH + ' - ' + HEIGHT);
    canvas.width = parent.clientWidth - 2 * BORDER;
    draw();
}

function init() {
    parent.style.paddingLeft = BORDER + 'px';
    parent.style.paddingRight = BORDER + 'px';
    onWindowResize();
}

function exchange() {
    if (T > 0) {
        let interval = setInterval(doStuff, 50);
        function doStuff() {
            T -= 0.1;
            if (T < 0) {
                clearInterval(interval);
                T = 0;
            }
            draw();
        }
    }
    else {
        let interval = setInterval(doStuff, 50);
        function doStuff() {
            T += 0.1;
            if (T > 1) {
                clearInterval(interval);
                T = 1;
            }
            draw();
        }
    }
}

function openFile() {
    FILE_INPUT.click();
}










window.addEventListener('load', init, false);
window.addEventListener('resize', onWindowResize, false);
document.getElementById('exchange').addEventListener('click', exchange, false);
document.getElementById('open').addEventListener('click', openFile, false);
document.getElementById('size').addEventListener('change', draw, false);

FILE_INPUT.addEventListener('change', function() {
    let file = FILE_INPUT.files[0];
    //console.log(file);
    if (file.type.indexOf('image') == 0)
        FILE_READER.readAsDataURL(file);
});

FILE_READER.addEventListener('load', function() {
    document.getElementById('messages').innerHTML = '<img id="myImg" src="' + FILE_READER.result + '" />';
    setTimeout(draw, 100);
});

